searchData={"items":[{"type":"module","title":"Mozart.BpmProcess","doc":"This module implements a BPM Domain Specific Language (DSL) framework. To use, insert\n**use Mozart.BpmProcess** into your modules.\n\n```elixir\ndefmodule Mozart.MyBpmApplication do\n  use Mozart.BpmProcess\n\n  # Your process model definitions\nend\n```","ref":"Mozart.BpmProcess.html"},{"type":"function","title":"Mozart.BpmProcess.assign_events/2","doc":"","ref":"Mozart.BpmProcess.html#assign_events/2"},{"type":"macro","title":"Mozart.BpmProcess.case_i/2","doc":"Used to specify one of many alternate execution paths. The first argument is either\na captured function or an atom corresponding to local function of arity 1. The second\nargument is a block of tasks.\n\n```\ndefprocess \"two case process\" do\n  case_task \"yes or no\" do\n    case_i :x_less_than_y do\n      user_task(\"1\", group: \"admin\")\n      user_task(\"2\", group: \"admin\")\n    end\n    case_i :x_greater_or_equal_y do\n      user_task(\"3\", group: \"admin\")\n      user_task(\"4\", group: \"admin\")\n    end\n  end\nend\n```","ref":"Mozart.BpmProcess.html#case_i/2"},{"type":"macro","title":"Mozart.BpmProcess.case_task/2","doc":"Used to select one of many execution paths. Arguments are a task name and a list of cases.\n\n```\ndefprocess \"two case process\" do\n  case_task \"yes or no\" do\n    case_i :x_less_than_y do\n      user_task(\"1\", group: \"admin\")\n      user_task(\"2\", group: \"admin\")\n    end\n    case_i :x_greater_or_equal_y do\n      user_task(\"3\", group: \"admin\")\n      user_task(\"4\", group: \"admin\")\n    end\n  end\nend\n```","ref":"Mozart.BpmProcess.html#case_task/2"},{"type":"macro","title":"Mozart.BpmProcess.conditional_task/3","doc":"Used to define a task that executes only when a condition (function)\nevaluates to true (a truthy value). When  the condition evaluates to\nfalse, execution proceeds directly to the next task.\n\nExample:\n\n```\ndefprocess \"process with conditional task\" do\n  prototype_task(\"fast initial credit check\")\n  conditional_task \"additional credit check\", condition: :initial_check_inconclusive do\n    prototype_task(\"extensive credit check\")\n  end\n  prototype_task(\"report credit check results\")\nend\n```","ref":"Mozart.BpmProcess.html#conditional_task/3"},{"type":"macro","title":"Mozart.BpmProcess.def_bpm_application/2","doc":"Used to define a BPM application. An BPM application is used to identity a single\nbusiness process implemented in a Elixir module. It's purpose is to facilitate external tool\nintegration. A single Elixir module might have zero or more such definitions. It's parameters\nare as follows:\n* **process**: The name of the top level process defintion.\n* **data**: A comma separated list of input parameters that the busines process should be\ninitialized with.\n\nExample:\ndef_bpm_application(\"Home Loan\", data: \"Customer Name,Income,Debt\")","ref":"Mozart.BpmProcess.html#def_bpm_application/2"},{"type":"macro","title":"Mozart.BpmProcess.def_choice_type/2","doc":"Used to associate a single choice, enumerated type for a process data parameter. It's\npurpose is to facilitate interation with external tools, e.g. GUIs.\n\nExample:\n\ndef_choice_type(\"Pre Approval\", choices: \"Approved, Declined\")","ref":"Mozart.BpmProcess.html#def_choice_type/2"},{"type":"macro","title":"Mozart.BpmProcess.def_confirm_type/1","doc":"Used to associate a confirmational type for a process data parameter. It's\npurpose is to facilitate interation with external tools, e.g. GUIs. Intended\nto be associated with a check box that the user must check to complete a task.\n\nExample:\n\ndef_confirm_type(\"confirm param\")","ref":"Mozart.BpmProcess.html#def_confirm_type/1"},{"type":"macro","title":"Mozart.BpmProcess.def_multi_choice_type/2","doc":"Used to associate a multiple choice, enumerated type for a process data parameter. It's\npurpose is to facilitate interation with external tools, e.g. GUIs.\n\nExample:\n\ndef_multi_choice_type(\"multi choice param\", choices: \"foo,bar,foobar\")","ref":"Mozart.BpmProcess.html#def_multi_choice_type/2"},{"type":"macro","title":"Mozart.BpmProcess.def_number_type/2","doc":"Used to associate a numerical type for a process data parameter. It's\npurpose is to facilitate interation with external tools, e.g. GUIs.\n\nExample:\n\ndef_number_type(\"number param\", min: 0, max: 5)","ref":"Mozart.BpmProcess.html#def_number_type/2"},{"type":"macro","title":"Mozart.BpmProcess.defevent/3","doc":"Used to implement a business process event. Arguments are:\n* the name of the event\n* **process**: the name of the process that the event will act upon\n* **exit_task**: the name of the task to be exited\n* **selector**: a function that matches on the target event\n* **do**: one or more tasks to be executed when the target task is exited.\n```\ndefevent \"exit loan decision 1\",\n  process: \"exit a user task 1\",\n  exit_task: \"user task 1\",\n  selector: :event_selector do\n    prototype_task(\"event 1 prototype task 1\")\n    prototype_task(\"event 1 prototype task 2\")\nend\n```","ref":"Mozart.BpmProcess.html#defevent/3"},{"type":"macro","title":"Mozart.BpmProcess.defprocess/2","doc":"Used to implement a business process model. Arguments are a name followed by one or\nmore task functions.\n\n```\ndefprocess \"two timer task process\" do\n  timer_task(\"one second timer task\", duration: 1000)\n  timer_task(\"two second timer task\", duration: 2000)\nend\n```","ref":"Mozart.BpmProcess.html#defprocess/2"},{"type":"function","title":"Mozart.BpmProcess.merge_event_tasks/2","doc":"","ref":"Mozart.BpmProcess.html#merge_event_tasks/2"},{"type":"macro","title":"Mozart.BpmProcess.parallel_task/2","doc":"Used to define parallel execution paths. Arguments are the task name\nand a list of routes.\n\n```\ndefprocess \"two parallel routes process\" do\n  parallel_task \"a parallel task\" do\n    route do\n      user_task(\"1\", group: \"admin\")\n      user_task(\"2\", group: \"admin\")\n    end\n    route do\n      user_task(\"3\", group: \"admin\")\n      user_task(\"4\", group: \"admin\")\n    end\n  end\nend\n```","ref":"Mozart.BpmProcess.html#parallel_task/2"},{"type":"macro","title":"Mozart.BpmProcess.prototype_task/2","doc":"Used to specify a task that has no behavior. Used for prototyping. Has a single\nname argument\n\n```\ndefprocess \"two prototype task process\" do\n  prototype_task(\"prototype task 1\")\n  prototype_task(\"prototype task 2\")\nend\n```","ref":"Mozart.BpmProcess.html#prototype_task/2"},{"type":"macro","title":"Mozart.BpmProcess.receive_task/2","doc":"Used to suspend an execution path until receiving a specified PubSub event. Arguemnts\nare a task name and captured function\n\n```\ndef receive_loan_income(msg) do\n  case msg do\n    {:barrower_income, income} -> %{barrower_income: income}\n    _ -> nil\n  end\nend\n\ndefprocess \"receive barrower income process\" do\n  receive_task(\"receive barrower income\", selector: :receive_loan_income)\nend\n```","ref":"Mozart.BpmProcess.html#receive_task/2"},{"type":"macro","title":"Mozart.BpmProcess.repeat_task/3","doc":"Used to a sequence of tasks that repeat as long as a condition evaluates to\na truthy value.\n\nExample:\n```\ndefprocess \"account overdue process\" do\n  repeat_task \"notify customer\", condition: :third_notice_sent do\n    service_task(\"send overdue notice\", function: :record_notifiction_sent)\n  end\nend\n```","ref":"Mozart.BpmProcess.html#repeat_task/3"},{"type":"macro","title":"Mozart.BpmProcess.reroute_task/3","doc":"Used to reroute a process flow off the typical execution path (off the 'happy path').\n\nExample:\n```\ndefprocess \"act on one of multiple events\" do\n  prototype_task(\"create order\")\n  receive_task(\"receive payment details\", selector: :receive_payment_details)\n  reroute_task \"payment period expired\", condition: :payment_period_expired do\n    prototype_task(\"cancel order due to timeout\")\n  end\n  reroute_task \"order canceled\", condition: :order_canceled do\n    prototype_task(\"cancel order due to order cancelation\")\n  end\n  prototype_task(\"process payment\")\nend\n```","ref":"Mozart.BpmProcess.html#reroute_task/3"},{"type":"macro","title":"Mozart.BpmProcess.rule_task/2","doc":"Used to set data properties based on evaluation of rules in a rule table. Arguments\nare a task name, a set of inputs and a rule table.\n\n```\nrule_table = \"\"\"\nF     income      || status\n1     > 50000     || approved\n2     <= 49999    || declined\n\"\"\"\n\ndefprocess \"single rule task process\" do\n  rule_task(\"loan decision\", inputs: \"income\", rule_table: rule_table)\nend\n```","ref":"Mozart.BpmProcess.html#rule_task/2"},{"type":"macro","title":"Mozart.BpmProcess.send_task/2","doc":"A task that send a PubSub event to a receiving receive_task. Arguments are a task name\nand a PubSub message.\n\n```\ndefprocess \"send barrower income process\" do\n  send_task(\"send barrower income\", message: {:barrower_income, 100_000})\nend\n```","ref":"Mozart.BpmProcess.html#send_task/2"},{"type":"macro","title":"Mozart.BpmProcess.service_task/2","doc":"Used to specify a task completed by calling an Elixir function. Arguments are\na task name, a set of comma delitmited inputs a captured Elixir function.\n\n```\ndef square(data) do\n  Map.put(data, :square, data.x * data.x)\nend\n\ndefprocess \"one service task process\" do\n  service_task(\"a service task\", function: :square, inputs: \"x\")\nend\n```","ref":"Mozart.BpmProcess.html#service_task/2"},{"type":"macro","title":"Mozart.BpmProcess.subprocess_task/2","doc":"Used to specify a task completed by spawning and completing a subprocess. Arguments\nare a task name and the name of the subprocess model.\n\n```\ndefprocess \"subprocess task process\" do\n  subprocess_task(\"subprocess task\", process: \"two service tasks\")\nend\n```","ref":"Mozart.BpmProcess.html#subprocess_task/2"},{"type":"macro","title":"Mozart.BpmProcess.timer_task/2","doc":"Used to specify a timed delay in an execution path. Arguments are a task name and\na duration in milliseconds.\n\n```\ndefprocess \"two timer task process\" do\n  timer_task(\"one second timer task\", duration: 1000)\n  timer_task(\"two second timer task\", duration: 2000)\nend\n```","ref":"Mozart.BpmProcess.html#timer_task/2"},{"type":"macro","title":"Mozart.BpmProcess.user_task/2","doc":"Used to specify a task performed by a user belonging to a specified workgroup. Arguments are\na task name and a set of comma delimited workgroups.\n\n```\ndefprocess \"single user task process\" do\n  user_task(\"add one to x\", group: \"admin,customer_service\")\nend\n```","ref":"Mozart.BpmProcess.html#user_task/2"},{"type":"module","title":"Mozart.Data.ProcessState","doc":"Defines a struct with fields specifying the state of a BPM process execution. The fields\nprovide the following information:\n\n* **uid**: A unique identifier of a single process model execution.\n* **business_key**: A unique identifier for a hierarchy of process model executions\ncorresponding to a hierarchial process model.\n* **parent_uid**: If the process is a child process, this is the pid of the parent process\nexecution.\n* **process**: The name of the process model being executed.\n* **start_time**: The time when the process started executing.\n* **end_time**: The time when the process finished executing.\n* **execution_duration**: The duration of time over which the process executed.\n* **open_tasks**: a map of open tasks. The key is a process name. The value is the\ninternal structure of the task.\n* **completed_tasks**: A list of completed tasks. List items are task internal\nstructures.\n* **data**: A map of all accumulated process data. Keys are the property names. Values\nare the property values.\n* **complete**: Indicates whether the process has completed.\n\n```\ndefstruct [\n    :uid,\n    :business_key,\n    :parent_uid,\n    :process,\n    :start_time,\n    :end_time,\n    :execute_duration,\n    open_tasks: %{},\n    completed_tasks: [],\n    data: %{},\n    complete: false\n  ]\nend\n```","ref":"Mozart.Data.ProcessState.html"},{"type":"module","title":"Mozart.ProcessEngine","doc":"A ProcessEngine is dynamically spawned for the purpose of executing a process model defined by **defprocess** function call.","ref":"Mozart.ProcessEngine.html"},{"type":"function","title":"Mozart.ProcessEngine.execute/1","doc":"Used to complete any \"complete-able\" open tasks. Task execution frequently spawns new\nopen tasks. Execute will continue to called recursively as long as there are \"complete-able\" open tasks.","ref":"Mozart.ProcessEngine.html#execute/1"},{"type":"function","title":"Mozart.ProcessEngine.get_open_tasks/1","doc":"Gets the open tasks of the given process engine. Useful for debugging.","ref":"Mozart.ProcessEngine.html#get_open_tasks/1"},{"type":"function","title":"Mozart.ProcessEngine.get_state/1","doc":"Returns the state of the process. Useful for debugging.","ref":"Mozart.ProcessEngine.html#get_state/1"},{"type":"function","title":"Mozart.ProcessEngine.start_process/5","doc":"Use this function to create a ProcessEngine instance initialized with the\nname of the process model to be executed and any initialization data. The\nengine will start executing tasks when the execute/1 function is called.\n\nArguments are a process model name, initial process data, an optional business_key, and a parent\nprocess pid if there is a parent. If a process key is not specified, one will be assigned.\n\nReturns a tuple of the form:\n```\n{:ok, ppid, uid, business_key}\n```\nwhere:\n* **ppid** is the Elixir pid for the spawned GenServer.\n* **uid** is a uniquie identifier for a process execution.\n* **business_key** is a unique identifier for a hierarchial process execution.\n\nSample invocation:\n```\n{:ok, ppid, uid, business_key} = ProcessEngine.start_process(\"a process model name\", )\n```","ref":"Mozart.ProcessEngine.html#start_process/5"},{"type":"module","title":"Mozart.ProcessService","doc":"This modeule provides a set of utility services.","ref":"Mozart.ProcessService.html"},{"type":"function","title":"Mozart.ProcessService.complete_user_task/2","doc":"Completes a user task.","ref":"Mozart.ProcessService.html#complete_user_task/2"},{"type":"function","title":"Mozart.ProcessService.get_active_processes/0","doc":"Get active process instances","ref":"Mozart.ProcessService.html#get_active_processes/0"},{"type":"function","title":"Mozart.ProcessService.get_bpm_application/1","doc":"Gets a BPM Application","ref":"Mozart.ProcessService.html#get_bpm_application/1"},{"type":"function","title":"Mozart.ProcessService.get_bpm_applications/0","doc":"Gets the names of all BPM applications","ref":"Mozart.ProcessService.html#get_bpm_applications/0"},{"type":"function","title":"Mozart.ProcessService.get_completed_process/1","doc":"Returns the state of the completed process corresponding to the process engine's uid","ref":"Mozart.ProcessService.html#get_completed_process/1"},{"type":"function","title":"Mozart.ProcessService.get_completed_process_data/1","doc":"Returns the data accumulated by the completed process","ref":"Mozart.ProcessService.html#get_completed_process_data/1"},{"type":"function","title":"Mozart.ProcessService.get_process_model/1","doc":"Retrieves a process model by name.","ref":"Mozart.ProcessService.html#get_process_model/1"},{"type":"function","title":"Mozart.ProcessService.get_user_tasks_for_group/1","doc":"Returns the user tasks that can be completed by users belonging to one of the input group.","ref":"Mozart.ProcessService.html#get_user_tasks_for_group/1"},{"type":"function","title":"Mozart.ProcessService.handle_continue/2","doc":"","ref":"Mozart.ProcessService.html#handle_continue/2"},{"type":"function","title":"Mozart.ProcessService.load_bpm_application/1","doc":"Loads a BPM Application","ref":"Mozart.ProcessService.html#load_bpm_application/1"},{"type":"function","title":"Mozart.ProcessService.load_process_model/1","doc":"Loads a single process model in the repository.","ref":"Mozart.ProcessService.html#load_process_model/1"},{"type":"function","title":"Mozart.ProcessService.load_process_models/1","doc":"Loads a list of process model into the state of the\nProcessService.","ref":"Mozart.ProcessService.html#load_process_models/1"},{"type":"extras","title":"Introduction to BPM","doc":"# Introduction to BPM\n\nThe next several sections provide an introduction to to ***Business Process Management***.","ref":"intro_bpm.html"},{"type":"extras","title":"What is a Business Process? - Introduction to BPM","doc":"Companies large and small have business processes that they regularly execute. Let's just call them \"processes\" for brevity's sake. Insurance companies have processes to process a claim. Banks have processes to approve loans. The government have processes to award contracts. \n\nBusiness processes have the following charateristics:\n\n* A business process is comprised of some number of individual tasks. \n* Tasks are performed in an order prescribed by network of task connections.\n* Processes can branch to multiple alternative sets of tasks depending on the state of the process.\n* Processes can branch to multiple parallel paths as well.\n* There are multiple kinds of tasks. Some tasks are performed by users. Some tasks are performed by calling services. There are many other kinds of tasks as well.\n* Processes store data to be used by tasks to perform their functions.\n* Processes allow tasks to add data for use by downstream tasks.","ref":"intro_bpm.html#what-is-a-business-process"},{"type":"extras","title":"What is Business Process Management (BPM)? - Introduction to BPM","doc":"Business Process Management (BPM) is the organized and directed activity of a company to control, monitor and improve the quality of their business processes. They do this with the aid of a BPM platform made available by dozens of commercial vendors.\n\nBPM platforms provide the following functions:\n\n* Provide developers with the ability of define and store process models.\n* Provide developers with the ability have certain tasks completed by making service calls.\n* Provide users and administrators with the ability to start process instances.\n* Provide users with the ability to assign user tasks to specified users.\n* Prvide users with the ability to assign user tasks to members of a specified user group.\n\nThe goals of Business Process Management are:\n\n* Better Process Reliablibility\n* Faster Process Execution\n* Reduced Training Requirements\n* Incremental Process Improvement\n* Provide Process Metrics\n* Cost Savings","ref":"intro_bpm.html#what-is-business-process-management-bpm"},{"type":"extras","title":"When is BPM Applicable - Introduction to BPM","doc":"This is difficult to answer exactly, but BPM may be applicable when:\n\n* Processes are performed on a regular basis.\n* Successful process execution is crucial to the business.\n* Current processes are error prone.\n* The speed of process completion is very important.\n* Processes are well defined.\n* Multiple business groups and/or people participate in process execution.","ref":"intro_bpm.html#when-is-bpm-applicable"},{"type":"extras","title":"Introduction to Mozart","doc":"# Introduction to Mozart","ref":"app_overview.html"},{"type":"extras","title":"Process Models - Introduction to Mozart","doc":"The primary concept of *Mozart* is the **Process Model**. A *process model* is an *executable* representation of a business process. *Process models* are implemented using a collection of functions that together comprise a **Domain Specific Language (DSL)** used for the purpose of creating BPM Applications.\n\nIf you are a developer working for a bank, you might be asked to develop a process model for handling a customer loan application. A grossly simplified process model for this might look like:\n\n```elixir\n  defprocess \"process loan application\" do\n    user_task(\"record and check loan application\",  group: \"customer service\")\n    user_task(\"determine loan acceptance\",          group: \"underwriting\")\n    user_task(\"inform customer of result\",          group: \"customer service\")\n  end\n```","ref":"app_overview.html#process-models"},{"type":"extras","title":"Process Model Tasks - Introduction to Mozart","doc":"As indicated in the example above, a *process model* is composed of a collection of **tasks**. Each task represents some work that needs to be completed. To **complete** a task means to perform the work required of the task.\n\nMozart provides around a dozen different types of *tasks*. Tasks are defined using functions in the Mozart DSL.\n\nThe task types currently supported are:\n\n| Task Type               |  Description |\n|-----|-----|\n| user_task               | Performed by a user |\n| service_task            | Performed by calling a service. |\n| subprocess_task         | Performed by calling a subprocess. |\n| timer_task              | Waits for expiration of a timer. |\n| receive_task            | Waits for a subscribed PubSub event. |\n| send_task               | Sends a PubSub event. |\n| rule_task               | Performed by evaluating a set of rules represented in a table. |\n| case_task               | Selects one of many process paths. |\n| parallel_task           | Initiates two or more parallel process paths. |\n| prototype_task          | Has no behavior. Used for prototyping and subbing. |\n| repeat_task             | Repeats a set of tasks while condition is true. |\n| reroute_task            | Reroutes a process off the typical (happy) execution path |\n| conditional_task        | Implements a conditional task along the typical execution path |","ref":"app_overview.html#process-model-tasks"},{"type":"extras","title":"BPM Process Module - Introduction to Mozart","doc":"A **BPM Process Module** is an Elixir module containing a set of related **process models, BPM applications, BPM related type definitions** as well as any other normal function definitions. There is an [example process application](https://github.com/CharlesIrvineKC/mozart/blob/main/test/support/home_loan_app.ex) in the Mozart GitHub repository.\n\n*Process applications* must **use Mozart.BpmProcess** to gain access to the Mozart DSL:\n\n```elixir\ndefmodule MyProcessApplication do\n  use Mozart.BpmProcess\n\n  defun send_loan_response(data) do\n    # Code to send customer and email loan resonse\n  end\n\n  defprocess \"process loan application\" do\n    user_task(\"record and check loan application\", group: \"customer service\")\n    user_task(\"determine loan acceptance\", group: \"underwriting\")\n    service_task(\"inform customer of result\", function: &MyProcessApplication.send_loan_response/1)\n  end\n\n  # multiple process model and function definitions\n\nend\n```\n\nDeclaring **use Mozart.BpmProcess** injects the **MyBpmApplication.load/0** function into your application. Calling this function will load all of your process models into the `Mozart.ProcessService` model repository. Use it now to load your process models:\n\n```elixir\nMyProcessApplication.load()\n```","ref":"app_overview.html#bpm-process-module"},{"type":"extras","title":"BPM Applications - Introduction to Mozart","doc":"Each Elixir BPM module can define zero or more **BPM Applications**. Here is an example\n\n```elixir\ndef_bpm_application(\"Home Loan\", data: \"Customer Name,Income,Debt\")\n```\nEach application specifies a user level name, the name of the top level process definition and a set of required \ninput parameters.\n\nThe purpose of a BPM application is to aid in intergration with external applications, e.g. GUI applications.","ref":"app_overview.html#bpm-applications"},{"type":"extras","title":"Parameter type specifications - Introduction to Mozart","doc":"Each Elixir BPM process module can define any number of parameter type definitions. Here are examples\nof the currently supported types:\n\n```elixir\ndef_number_type(\"number param\", min: 0, max: 5)\ndef_choice_type(\"choice param\", choices: \"foo, bar\")\ndef_multi_choice_type(\"multi choice param\", choices: \"foo,bar,foobar\")\ndef_confirm_type(\"confirm param\")\n```\n\nType definitions aid in integration with external applications, e.g. GUIs.","ref":"app_overview.html#parameter-type-specifications"},{"type":"extras","title":"Process Engine - Introduction to Mozart","doc":"We have said that a Mozart process model is **executable**. What does that mean? After loading the process model as shown above, commands are invoked to start and begin executing the process model. Here is a corresponding code snippet:\n\n```elixir\ndata = %{loan_amount_requested: 300_000}\n\n{:ok, ppid, uid, _business_key} = ProcessEngine.start_process(\"process loan application\", data)\n\nProcessEngine.execute(ppid)\n```\n\nFirst, we define some data that the process model will use. Then a *process engine* is spawned by calling **ProcessEngine.start_process/2**. The first parameter specifies the top level *process model* to be executed. As you will see later on, a hierarchy of process instances can be spawned to complete the top level process model. The second argument is data used to initiate process execution. \n\nThe first argument returned is the Elixir *process identifier* of the Elixir GenServer. The secord value returned is a unique identifier of the process execution. This will be useful beyond the lifetime of the Elixir process. Finally, the third value returned is another unique identifier corresponding to the hierarchy of processes spawned to complete the top level process model.\n\nAfter, the process engine has been spawned, process execution begins by calling **ProcessEngine.execute/1** function, passing it the Elixir PID contained in the return value bound to the variable **ppid**.","ref":"app_overview.html#process-engine"},{"type":"extras","title":"Process Service - Introduction to Mozart","doc":"We have already seen that the GenServer `Mozart.ProcessService` provides a repository for process models. It provides numerous additional functions [documented in hexdocs](https://hexdocs.pm/mozart/Mozart.ProcessService.html#summary).","ref":"app_overview.html#process-service"},{"type":"extras","title":"Processes Model Execution - Introduction to Mozart","doc":"Earlier we said that the job of the process engine is to drive a process model execution to completion. Let's discuss what that means.","ref":"app_overview.html#processes-model-execution"},{"type":"extras","title":"Completing Tasks - Introduction to Mozart","doc":"When process execution is triggered by calling **ProcessEngine.execute/1**, the first task specified in the process model will be opened for completion.\n\n```elixir\n defprocess \"process loan application\" do\n    user_task(\"record and check loan application\", group: \"customer service\")\n    user_task(\"determine loan acceptance\", group: \"underwriting\")\n    service_task(\"inform customer of result\", function: &MyProcessApplication.send_loan_response/1)\n  end\n```\n\nHence, in the process model above, the user task named **record and check loan application** will be opened for completion upon the initiation of process execution.\n\nCompletion of this initial task will typically trigger the opening of additional tasks. Every task that completes will potentially trigger the opening of additional tasks. Eventually, a process model will have no open tasks. This signals that execution of the process model is complete and the corresponding process engine will exit.\n\nSo, completing the execution of a process model equates to repeatedly opening and completing tasks until the process as a whole is complete.","ref":"app_overview.html#completing-tasks"},{"type":"extras","title":"Accumulating Data - Introduction to Mozart","doc":"Almost every task that is opened will need some data to complete. Further, when a task completes, it will introduce some new data for downstream tasks to use. Thus, to a great extent, completing a processes equates to generating a store of accumulated information.\n\nAll of the data generated by task completion is stored in the **data** property of `Mozart.ProcessEngine`.","ref":"app_overview.html#accumulating-data"},{"type":"extras","title":"Producing Side Effects - Introduction to Mozart","doc":"To be useful, almost all process model executions will produce some side effects. Take the process of processing a bank loan application for example. What are the possible side effects?\n\n* Sending a notice that the loan is approved or declined.\n* Depositing money in the customers account.\n* Setting up the loan in the banks backend system.","ref":"app_overview.html#producing-side-effects"},{"type":"extras","title":"Conclusion - Introduction to Mozart","doc":"Hopefully, the mechanics of Business Process Managment is now much clearer. In the newxt section, we will use Mozart to execute business processes.","ref":"app_overview.html#conclusion"},{"type":"extras","title":"First Process Execution","doc":"# First Process Execution\n\nNow we will put together what we've lerned so far to execute a process model.","ref":"first_process_execution.html"},{"type":"extras","title":"To Follow Along.. - First Process Execution","doc":"To follow along create a new Elxir **mix** project and then add Mozart as a dependency in your **mix.exs** project file. Your dependency should look like this:\n\n```elixir\n  defp deps do\n    [\n      {:mozart, \"~> 0.4\"}\n    ]\n  end\n```","ref":"first_process_execution.html#to-follow-along"},{"type":"extras","title":"Create BPM Application - First Process Execution","doc":"First you need to create a BPM application module. In it you will specify your process models and any needed supported functions. Add the following file to your project.\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess\n\n  # Your process models and supporting functions will go here.\n\nend\n```","ref":"first_process_execution.html#create-bpm-application"},{"type":"extras","title":"Construct the Process Model - First Process Execution","doc":"First, let us construct a simple process model that will have a single service task. The service task will simply return the sum of two input properties. Update your BPM application module as follows:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess\n\n  def sum(data) do\n    %{sum: data.x + data.y}\n  end\n\n  defprocess \"add x and y process\" do\n    service_task(\"add x and y task\", function: &MyBpmApplication.sum/1, inputs: \"x,y\")\n  end\n\nend\n```","ref":"first_process_execution.html#construct-the-process-model"},{"type":"extras","title":"Load the Process Model into repository - First Process Execution","doc":"Now let's load this process model into the process model repository, which is implemented by `Mozart.ProcessService`. First, start an iex session with **iex -S mix** and paste a couple of aliases into your iex session:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\n\n```\n\nNow we are ready to load our process model into the repository. Copy the following into your iex session:\n\n```elixir\nMyBpmApplication.load()\n\n```\n\nYou should see:\n\n````\niex [15:06 :: 3] > MyBpmApplication.load()\n{:ok, [\"add x and y process\"]}\n````\n\nThe function **MyBpmApplication.load/0** was created for you automatically when you inserted **use Mozart.BpmProcess** into your module definition.","ref":"first_process_execution.html#load-the-process-model-into-repository"},{"type":"extras","title":"Start a Process Engine and Execute the Process Model - First Process Execution","doc":"Now let us start a process engine, initializing it with our process model and some data. Then we call the function that will invoke process execution. Copy the following into your iex session:\n\n```elixir\n{:ok, ppid, uid, _key} = PE.start_process(\"add x and y process\", %{x: 1, y: 1})\n\n```\n\nYou should now see something like:\n\n```\n10:04:26.242 [info] Start process instance [add x and y process][a57d93d4-fc97-4a3c-89f6-989c088c96a7]\n{:ok, #PID<0.457.0>, \"a57d93d4-fc97-4a3c-89f6-989c088c96a7\", \"d41dc645-2417-4778-bb18-fccf2e6e44f6\"}\n```\n\nWe started a process instance using `Mozart.ProcessEngin.start_process/2`. We passed it the name of the process model to be executed and some initial data required for process execution. The function call returned a tuple with three values: \n\n* The **pid** of the resulting process engine.\n* The **uid** of the process instance. Every process execution will be given a unique identifier. You will see how this is used going forward.\n* Finally, a **business_key** is returned. Process models are hierarchial, meaning they can be composed of multiple, nested subprocesses. The top level business process and all of its subprocesses will share the same *business_key*. Here, were aren't going to be using the *process key*, but we will in subsequent examples.\n\nAfter creating the process engine instance, we then call `Mozart.ProcessEngine.execute/1` which causes the process engine to start completing any tasks that are ready to be completed. Let's do that now:\n\n```\nPE.execute(ppid)\n\n```\n\nAfter calling the *ProcessEngine.execute* function, you should see something like:\n\n```elixir\niex [08:50 :: 23] > PE.execute(ppid)\n:ok\n10:02:40.292 [info] New service task instance [add x and y task][ddd36158-7837-4c5a-b0be-66a02f304d27]\n10:02:40.292 [info] Complete service task [add x and y task[ddd36158-7837-4c5a-b0be-66a02f304d27]\n10:02:40.292 [info] Process complete [add x and y process][a1d4525d-96cb-4da1-9a82-d20bab4afa89]\n```\n\nThe logs show that:\n\n1. A new service task was opened.\n1. The service task completed.\n1. And finally the process engine completed executing the process model.","ref":"first_process_execution.html#start-a-process-engine-and-execute-the-process-model"},{"type":"extras","title":"Verify the Results - First Process Execution","doc":"Let's verify the results of our process model execution by calling **ProcessService.get_completed_process_data/1**. Copy the following into your iex session:\n\n```elixir\nPS.get_completed_process_data(uid)\n\n```\n\nand you should see:\n\n```elixir\niex [08:50 :: 26] > PS.get_completed_process_data(uid)\n%{sum: 2, y: 1, x: 1}\n```\n\nWe see that a **sum** property was added to process data whose value is the sume of the properties **x** and **y**..","ref":"first_process_execution.html#verify-the-results"},{"type":"extras","title":"Mozart Task Types","doc":"# Mozart Task Types\n\nIn this guide we will provide examples of all of the Mozart BPM DSL functions.\n\nIf you want to follow along, create an Elixir mix project that has Mozart as a dependency.\n\n```elixir\n  defp deps do\n    [\n      {:mozart, \"~> 0.4\"}\n    ]\n  end\n```\n\nNow, create an Elixir module named my_bpm_application.ex with the following contents:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess\n\n  # All of the examples will go here\n\nend\n\n```","ref":"task_types.html"},{"type":"extras","title":"Service Task - Mozart Task Types","doc":"A **Service Task** performs its work by calling an Elixir function. This function could perform a computation, call an external JSON service, retrieve data from a database, etc.\n\nA service task has two unique arguments: **function** and **inputs**.\n\n* The **function** field specifies the function that the service task should apply for the purpose of returning output data into the process state. The function can be either a captured function or an atom corresponding to a locally\ndefined function of arity 1.\n* The **inputs** field is used to select which process data fields are passed to the task's function. If no value is supplied for this field, the entire process data is passed.","ref":"task_types.html#service-task"},{"type":"extras","title":"Service Task example - Mozart Task Types","doc":"In your MyBpmApplication module, enter the following code:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess","ref":"task_types.html#service-task-example"},{"type":"extras","title":"Service Task Example - Mozart Task Types","doc":"def sum(data) do\n    %{sum: data.x + data.y}\n  end\n\n  defprocess \"add x and y process\" do\n    service_task(\"add x and y task\", function: &MyBpmApplication.sum/1, inputs: \"x,y\")\n    # You could also specify:\n    # service_task(\"add x and y task\", function: :sum, inputs: \"x,y\")\n  end\n\nend\n\n```\n\nWe've defined a process model that has a single service task. The name of the task is **\"add x and y task\"**. It will apply the function **MyBpmApplication.sum/1** to the arguments **x** and **y**.\n\nNow open an iex session on your project:\n\n```elixir\n$ iex -S mix\n\n```\n\nNow, paste in a couple of alias's:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\n\n```\n\nNow, you need to load your process model into the system. Paste in the following:\n\n```elixir\nMyBpmApplication.load()\n\n```\n\nwhich gives the result:\n\n```elixir\niex [16:41 :: 1] > MyBpmApplication.load()\n{:ok,\n [\"add x and y process\"]}\n```\n\n**MyBpmApplication.load/0** loads your BPM process application models into the model repository. You didn't define this function in your module. It was defined for you when you added **use Mozart.BpmProcess** to your module.\n\nNow you are ready to start a **ProcessEngine** that will run your process model:\n\n```elixir\n{:ok, ppid, uid, business_key} = PE.start_process(\"add x and y process\", %{x: 1, y: 1})\n\n```\n\nYou should then see:\n\n```elixir\n08:33:47.895 [info] Start process instance [add x and y process][299f3213-94ab-418d-bee3-18da04f2c38d]\n{:ok, #PID<0.302.0>, \"299f3213-94ab-418d-bee3-18da04f2c38d\",\n \"4d6e7fba-b303-4c9b-8cf5-a291168bd73d\"}\n```\n\nYour process engine is now ready to execute your process model. Run the following function:\n\n```elixir\niex [08:21 :: 5] > PE.execute(ppid)\n:ok\n08:37:46.418 [info] New service task instance [add x and y task][211308eb-fc1f-45f9-b758-077b12e920db]\n08:37:46.418 [info] Complete service task [add x and y task[211308eb-fc1f-45f9-b758-077b12e920db]\n08:37:46.418 [info] Process complete [add x and y process][299f3213-94ab-418d-bee3-18da04f2c38d]\n```\n\nAs the log entires indicate:\n\n* A new service task was created.\n* The service task completed it work (by applying the function you specified).\n* And finally the process completed.\n\nTo verify, that your process did what it was supposed to do, execute the following:\n\n```elixir\niex [08:21 :: 6] > PS.get_completed_process_data(uid)\n%{sum: 2, y: 1, x: 1}\n```","ref":"task_types.html#service-task-example"},{"type":"extras","title":"User Task - Mozart Task Types","doc":"A **User Task** works by having a user complete some required task. This often takes the form of a user using a GUI to examine a subset of process data and then supplying additional data. \n\n**Important Note**: Users interact with a BPM platfrom such as Mozart by way of end user GUI application. The user application will allow the user to find tasks appropriate to his role. Once the user accepts responsibility for a task, the application will then provide an input form appropriate for accomplishing the given task. For now, we will use Mozart functions for finding and completing tasks.\n\nA user task has two unique arguments: **group** and **:inputs**.\n\n* The **group** field specifies the user group in the form of a comma separated list that are elibigle to complete the task. \n* The **inputs** field is used to select which process data fields are passed to the user that will compleete the task. If no value is supplied for this field, the entire process data is passed.","ref":"task_types.html#user-task"},{"type":"extras","title":"User Task example - Mozart Task Types","doc":"First, let's add a process model containing a user task;\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess","ref":"task_types.html#user-task-example"},{"type":"extras","title":"Previous Examples Omitted - Mozart Task Types","doc":"","ref":"task_types.html#previous-examples-omitted"},{"type":"extras","title":"User Task Example - Mozart Task Types","doc":"defprocess \"one user task process\" do\n    user_task(\"user gives sum of x and y\", group: \"admin\", inputs: \"x,y\")\n  end\n\nend\n\n```\n\nNow we drop back into iex:\n\n```\niex -S mix\n```\n\nNow we paste in code to define alias's, load our process models, start a process engine and then execute our process model.\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nMyBpmApplication.load()\n{:ok, ppid, uid, business_key} = PE.start_process(\"one user task process\", %{x: 1, y: 1})\nPE.execute(ppid)\n\n```\n\nand we should see:\n\n```elixir\nInteractive Elixir (1.16.2) - press Ctrl+C to exit (type h() ENTER for help)\niex [10:39 :: 1] > alias Mozart.ProcessEngine, as: PE\nMozart.ProcessEngine\niex [10:39 :: 2] > alias Mozart.ProcessService, as: PS\nMozart.ProcessService\niex [10:39 :: 3] > MyBpmApplication.load()\n{:ok, [\"add x and y process\", \"one user task process\"]}\niex [10:39 :: 4] > {:ok, ppid, uid, business_key} = PE.start_process(\"one user task process\", %{x: 1, y: 1})\n\n10:39:49.816 [info] Start process instance [one user task process][714ff531-762f-4677-a357-dda88d7fb1c1]\n{:ok, #PID<0.285.0>, \"714ff531-762f-4677-a357-dda88d7fb1c1\",\n \"75a5124a-a51c-446f-80e7-d253dd3b5bc9\"}\niex [10:39 :: 5] > PE.execute(ppid)\n:ok\n\n10:39:49.824 [info] New user task instance [user gives sum of x and y][3fc1cdf8-4049-40db-9172-9a7f58a85cb8]\n```\n\nThe last log message tells us that a new user task was opened, but we don't see logs indicating that the task completed or that the process completed. That is because a user task waits to be completed by a user.\n\nTo complete the user task we need the task uid, which was printed in the last log statement:\n\n```elixir\n10:39:49.824 [info] New user task instance [user gives sum of x and y][3fc1cdf8-4049-40db-9172-9a7f58a85cb8]\n```\n\nIt is the value in last pair of square brackets: \"3fc1cdf8-4049-40db-9172-9a7f58a85cb8\". \n\nThe final thing we need to complete the task is the data that we want to merge into the process data. We want to merge in the sum of the inputs **x** and **y**. A GUI application for completing user tasks would have access to this data and would make it available to the user completing the task. So, let's assume that we know the value of our inputs and we use them to complete our task.\n\nLet's do that now by calling ProcessService.complete_user_task/2. \n\n```elixir\nPS.complete_user_task(\"3fc1cdf8-4049-40db-9172-9a7f58a85cb8\", %{sum: 3})\n\n```\n\nand we should see:\n\n```elixir\niex [13:05 :: 8] > PS.complete_user_task(\"7f3d7009-ab44-4955-a501-677e0a8a353b\", %{sum: 3})\n:ok\n13:16:21.326 [info] Complete user task [user gives sum of x and y][7f3d7009-ab44-4955-a501-677e0a8a353b]\n13:16:21.327 [info] Process complete [one user task process][fca5f283-0992-4921-ba16-f9d902f9e403]\n```","ref":"task_types.html#user-task-example"},{"type":"extras","title":"Subprocess Task Example - Mozart Task Types","doc":"The **Subprocess Task** is completed by calling a subprocess. When subprocess completes, the corresponding subprocess task also completes.\n\nUpdate your MyBpmApplication module with the following code:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess","ref":"task_types.html#subprocess-task-example"},{"type":"extras","title":"Previous Code Here - Mozart Task Types","doc":"","ref":"task_types.html#previous-code-here"},{"type":"extras","title":"Subprocess Task Example - Mozart Task Types","doc":"def add_one_to_value(data) do\n    Map.put(data, :value, data.value + 1)\n  end\n\n  defprocess \"two service tasks\" do\n    service_task(\"service task 2\", function: &MyBpmApplication.add_one_to_value/1, inputs: \"value\")\n    service_task(\"service task 3\", function: &MyBpmApplication.add_one_to_value/1, inputs: \"value\")\n  end\n\n  defprocess \"subprocess task process\" do\n    service_task(\"service task 1\", function: &MyBpmApplication.add_one_to_value/1, inputs: \"value\")\n    subprocess_task(\"subprocess task\", process: \"two service tasks\")\n  end\n\nend\n\n```\n\nNow, we will open an iex session and to the following:\n\n1. Paste in our alias'.\n1. Load our process models.\n1. Start a process engine with the process named \"subprocess task process\".\n1. Execute the process.\n\nOpen your iex session and paste in:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nMyBpmApplication.load()\n{:ok, ppid, uid, business_key} = PE.start_process(\"subprocess task process\", %{value: 0})\nPE.execute(ppid)\n\n```\n\nAnd you should see something like:\n\n```elixir\niex [13:50 :: 1] > alias Mozart.ProcessEngine, as: PE\nMozart.ProcessEngine\niex [13:50 :: 2] > alias Mozart.ProcessService, as: PS\nMozart.ProcessService\niex [13:50 :: 3] > MyBpmApplication.load()\n{:ok,\n [\"add x and y process\", \"one user task process\", \"two service tasks\",\n  \"subprocess task process\"]}\niex [13:50 :: 4] > {:ok, ppid, uid, business_key} = PE.start_process(\"subprocess task process\", %{value: 0})\n\n13:50:13.910 [info] Start process instance [subprocess task process][903561d2-2f27-4014-9bf9-4334a0d93466]\n{:ok, #PID<0.288.0>, \"903561d2-2f27-4014-9bf9-4334a0d93466\",\n \"d16c2892-7419-4381-b060-e4d899d24ee6\"}\niex [13:50 :: 5] > PE.execute(ppid)\n:ok\n13:50:13.912 [info] New service task instance [service task 1][ee364170-99ad-4ff9-8eda-e4c598e3d7da]\n13:50:13.912 [info] Complete service task [service task 1[ee364170-99ad-4ff9-8eda-e4c598e3d7da]\n13:50:13.913 [info] New sub_process task instance [subprocess task][251b46d7-bf0e-4818-bf28-45a628b85087]\n13:50:13.913 [info] Start process instance [two service tasks][cd9cb03f-f50c-48f2-ba51-9ed4d31c18fa]\n13:50:13.913 [info] New service task instance [service task 2][1f09acb8-b1ef-465e-9fe3-8f9a754f55e2]\n13:50:13.913 [info] Complete service task [service task 2[1f09acb8-b1ef-465e-9fe3-8f9a754f55e2]\n13:50:13.913 [info] New service task instance [service task 3][31bc8c0b-cd65-4e80-9afe-74fa546c7412]\n13:50:13.913 [info] Complete service task [service task 3[31bc8c0b-cd65-4e80-9afe-74fa546c7412]\n13:50:13.913 [info] Complete subprocess task [subprocess task][251b46d7-bf0e-4818-bf28-45a628b85087]\n13:50:13.913 [info] Process complete [two service tasks][cd9cb03f-f50c-48f2-ba51-9ed4d31c18fa]\n13:50:13.913 [info] Process complete [subprocess task process][903561d2-2f27-4014-9bf9-4334a0d93466]\n```\n\nIf we examine the logs, we should see that what we expect to happen did happen.\n\nWhen top level process finishes, we would expect the **value** parameter would have a value of three. Let's check if that is the case:\n\n```elixir\niex [13:50 :: 6] > PS.get_completed_process_data(uid)\n%{value: 3}\n```","ref":"task_types.html#subprocess-task-example"},{"type":"extras","title":"Case Task Example - Mozart Task Types","doc":"The **Case Task** provides a way to specify alternate execution paths depending on the current state of process execution.\n\nAdd the following content to the MyBpmApplication module:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess","ref":"task_types.html#case-task-example"},{"type":"extras","title":"Previous content here - Mozart Task Types","doc":"","ref":"task_types.html#previous-content-here"},{"type":"extras","title":"Case Task Example - Mozart Task Types","doc":"def add_two_to_value(data) do\n    Map.put(data, :value, data.value + 2)\n  end\n\n  def subtract_two_from_value(data) do\n    Map.put(data, :value, data.value - 2)\n  end\n\n  def x_less_than_y(data) do\n    data.x  = data.y\n  end\n\n  defprocess \"two case process\" do\n    case_task \"yes or no\" do\n      case_i &MyBpmApplication.x_less_than_y/1 do\n        service_task(\"1\", function: &MyBpmApplication.subtract_two_from_value/1, inputs: \"value\")\n        service_task(\"2\", function: &MyBpmApplication.subtract_two_from_value/1, inputs: \"value\")\n      end\n      case_i :x_greater_or_equal_y do\n        service_task(\"3\", function: &MyBpmApplication.add_two_to_value/1, inputs: \"value\")\n        service_task(\"4\", function: &MyBpmApplication.add_two_to_value/1, inputs: \"value\")\n      end\n    end\n  end\n\nend\n```\n\nThe first argument to **case_task** is a task name and the second argument is a block of cases.\n\nThe first argument to **case_i** is either a captured function of arity 1 or an atom corresponding to a locally defined function of arity 1. The second argument is a block of tasks.\n\nThe process named \"two case process\" performs this logic:\n\n* If **x is less than y**, subtract 2 from the value parameter twice.\n* if **x is greater or equal to y**, add 2 to the value parameter twice.\n\nLet's try it out. Open an iex session, and paste in the following:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nMyBpmApplication.load()\n{:ok, ppid, uid, business_key} = PE.start_process(\"two case process\", %{x: 1, y: 2, value: 10})\nPE.execute(ppid)\n\n```\nand you should see:\n\n```elixir\niex [14:33 :: 1] > alias Mozart.ProcessEngine, as: PE\nMozart.ProcessEngine\niex [14:33 :: 2] > alias Mozart.ProcessService, as: PS\nMozart.ProcessService\niex [14:33 :: 3] > MyBpmApplication.load()\n{:ok,\n [\"add x and y process\", \"one user task process\", \"two service tasks\",\n  \"subprocess task process\", \"two case process\"]}\niex [14:33 :: 4] > {:ok, ppid, uid, business_key} = PE.start_process(\"two case process\", %{x: 1, y: 2, value: 10})\n\n14:33:41.084 [info] Start process instance [two case process][4fb6889e-250c-407e-a332-f904f491b39d]\n{:ok, #PID<0.285.0>, \"4fb6889e-250c-407e-a332-f904f491b39d\",\n \"f5a76516-1509-4e01-99dd-949dba2c06c4\"}\niex [14:33 :: 5] > PE.execute(ppid)\n:ok\n14:33:41.088 [info] New case task instance [yes or no][4a7fd5c1-eaa5-4cfc-9edf-73bd28eaa9cb]\n14:33:41.088 [info] Complete case task [yes or no][4a7fd5c1-eaa5-4cfc-9edf-73bd28eaa9cb]\n14:33:41.088 [info] New service task instance [1][6482abb3-c56f-43d7-870f-83789b77135b]\n14:33:41.088 [info] Complete service task [1[6482abb3-c56f-43d7-870f-83789b77135b]\n14:33:41.089 [info] New service task instance [2][7ee68e1e-9823-4e92-9613-e34f1c279c3a]\n14:33:41.089 [info] Complete service task [2[7ee68e1e-9823-4e92-9613-e34f1c279c3a]\n14:33:41.089 [info] Process complete [two case process][4fb6889e-250c-407e-a332-f904f491b39d]\n```\n\nBased on our input data, since x was less than y, 2 should have been subtracted from 10 twice, leaving the **value** parameter with a value of 6. Let's see if that matches our result:\n\n```elixir\niex [13:50 :: 6] > PS.get_completed_process_data(uid)\n%{value: 6, y: 2, x: 1}\n```","ref":"task_types.html#case-task-example"},{"type":"extras","title":"Send and Receive Task Example - Mozart Task Types","doc":"For this example, we will use both the Send and Receive tasks since they are typically used together, that is, a receive task receives a messge from a send task.\n\nUpdate the MyBpmApplication module with the following content:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess","ref":"task_types.html#send-and-receive-task-example"},{"type":"extras","title":"Previous Content Here - Mozart Task Types","doc":"","ref":"task_types.html#previous-content-here"},{"type":"extras","title":"Send and Receive Example - Mozart Task Types","doc":"def receive_loan_income(msg) do\n    case msg do\n      {:barrower_income, income} -> %{barrower_income: income}\n      _ -> nil\n    end\n  end\n\n  defprocess \"receive barrower income process\" do\n    receive_task(\"receive barrower income\", selector: &MyBpmApplication.receive_loan_income/1)\n  end\n\n  defprocess \"send barrower income process\" do\n    send_task(\"send barrower income\", message: {:barrower_income, 100_000})\n  end\n\nend\n\n```\n\nIn this example we will start the \"receive barrower income process\" first. The \"receive barrower income\" task will wait until it receives a message of the form **{:barrower_income, income}**. When it receives this message it will merge **%{barrower_income: income}** into the processes data.\n\nThe purpose of the \"send barrower income process\" task is to send the message that the receive task is waiting for.\n\nOpen an iex session, and paste in the following:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nMyBpmApplication.load()\n{:ok, ppid, uid, business_key} = PE.start_process(\"receive barrower income process\", %{})\nPE.execute(ppid)\n\n```\n\nand you should see:\n\n```elixir\niex [17:23 :: 1] > alias Mozart.ProcessEngine, as: PE\nMozart.ProcessEngine\niex [17:23 :: 2] > alias Mozart.ProcessService, as: PS\nMozart.ProcessService\niex [17:23 :: 3] > MyBpmApplication.load()\n{:ok,\n [\"add x and y process\", \"one user task process\", \"two service tasks\",\n  \"subprocess task process\", \"two case process\",\n  \"receive barrower income process\", \"send barrower income process\"]}\niex [17:23 :: 4] > {:ok, ppid, uid, business_key} = PE.start_process(\"receive barrower income process\", %{})\n\n17:23:16.604 [info] Start process instance [receive barrower income process][d9534434-acc0-43a3-83ac-2f6db76cfb3d]\n{:ok, #PID<0.296.0>, \"d9534434-acc0-43a3-83ac-2f6db76cfb3d\",\n \"82d4ac32-e192-4a4b-a5e9-29f02c7ee46a\"}\niex [17:23 :: 5] > PE.execute(ppid)\n:ok\n\n17:23:16.608 [info] New receive task instance [receive barrower income][8286148c-a756-4db9-b21b-86acb3e8d17a]\n```\n\nAt this point, we've started the process with the receive task and we see that the expected receive task has been opened. Now we need to run the process with the send task so the waiting receive task can complete. To do that, copy the following into your iex session. \n\n```elixir\n{:ok, s_ppid, s_uid, s_business_key} = PE.start_process(\"send barrower income process\", %{})\nPE.execute(s_ppid)\n\n```\n\nNotice that when we call **PE.start_process/2**, we choose different varaible names so that our previous variable values won't be overwirtten.\n\n\nYou should see this result:\n\n```elixir\niex [17:23 :: 6] > {:ok, ppid, uid, business_key} = PE.start_process(\"send barrower income process\", %{})\n\n17:28:56.665 [info] Start process instance [send barrower income process][d58a5b10-1a12-48f5-9426-9ab90701e933]\n{:ok, #PID<0.299.0>, \"d58a5b10-1a12-48f5-9426-9ab90701e933\",\n \"7e9657d1-21ba-40e4-8c42-00f913c62926\"}\niex [17:23 :: 7] > PE.execute(ppid)\n:ok\n17:28:56.666 [info] New send task instance [send barrower income][273a9f29-783c-4306-97c4-eb9e12735a80]\n17:28:56.667 [info] Complete send event task [send barrower income[273a9f29-783c-4306-97c4-eb9e12735a80]\n17:28:56.667 [info] Complete receive event task [receive barrower income]\n17:28:56.667 [info] Process complete [send barrower income process][d58a5b10-1a12-48f5-9426-9ab90701e933]\n17:28:56.667 [info] Process complete [receive barrower income process][d9534434-acc0-43a3-83ac-2f6db76cfb3d]\n```\n\nNow we can check whether the expected data was merged into the process data of the first process:\n\n```elixir\niex [17:38 :: 8] > PS.get_completed_process_data(uid)\n%{barrower_income: 100000}\n```","ref":"task_types.html#send-and-receive-example"},{"type":"extras","title":"Parallel and Prototype Task Example - Mozart Task Types","doc":"A **Parallel Task** provides the ability to start two or more parallel execution paths. A **Prototype Task** has no behavior. A process engine will complete it automatically after it is opened. It is primarily used for stubbing tasks that will be replaced later with one of the other task types.\n\nCopy the following content into MyBpmApplication module:\n\n```elixir\ndefmodule MyBpmApplication do\n  @moduledoc false\n  use Mozart.BpmProcess","ref":"task_types.html#parallel-and-prototype-task-example"},{"type":"extras","title":"Previous Content Here - Mozart Task Types","doc":"","ref":"task_types.html#previous-content-here"},{"type":"extras","title":"Parallel and Prototype Task Example - Mozart Task Types","doc":"defprocess \"two parallel routes process\" do\n    parallel_task \"a parallel task\" do\n      route do\n        prototype_task(\"prototype task 1\")\n        prototype_task(\"prototype task 2\")\n      end\n      route do\n        prototype_task(\"prototype task 3\")\n        prototype_task(\"prototype task 4\")\n      end\n    end\n  end\n\nend\n\n```\n\nNow open an iex session and copy is the following:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nMyBpmApplication.load()\n{:ok, ppid, uid, business_key} = PE.start_process(\"two parallel routes process\", %{})\nPE.execute(ppid)\n\n```\n\nYou should see:\n\n```elixir\niex [18:29 :: 1] > alias Mozart.ProcessEngine, as: PE\nMozart.ProcessEngine\niex [18:29 :: 2] > alias Mozart.ProcessService, as: PS\nMozart.ProcessService\niex [18:29 :: 3] > MyBpmApplication.load()\n{:ok,\n [\"add x and y process\", \"one user task process\", \"two service tasks\",\n  \"subprocess task process\", \"two case process\",\n  \"receive barrower income process\", \"send barrower income process\",\n  \"two parallel routes process\"]}\niex [18:29 :: 4] > {:ok, ppid, uid, business_key} = PE.start_process(\"two parallel routes process\", %{})\n\n18:29:26.105 [info] Start process instance [two parallel routes process][08dba7e3-820e-4ea4-985e-5184b8041c80]\n{:ok, #PID<0.323.0>, \"08dba7e3-820e-4ea4-985e-5184b8041c80\",\n \"f4753b53-68ad-404b-ae0c-27bc9737b032\"}\niex [18:29 :: 5] > PE.execute(ppid)\n:ok\n18:29:26.108 [info] New parallel task instance [a parallel task][f7aa1870-e01a-46fb-8e0e-9a4353001241]\n18:29:26.108 [info] Complete parallel task [a parallel task]\n18:29:26.108 [info] New prototype task instance [prototype task 1][609a01d8-c7bd-4539-ad4e-720951ab56a3]\n18:29:26.109 [info] New prototype task instance [prototype task 3][16cc3d11-d8cb-4111-9b53-439a4cb10b94]\n18:29:26.109 [info] Complete prototype task [prototype task 3]\n18:29:26.109 [info] New prototype task instance [prototype task 4][e03c4cb7-e16d-45d4-98c1-4380bd57d240]\n18:29:26.109 [info] Complete prototype task [prototype task 1]\n18:29:26.109 [info] New prototype task instance [prototype task 2][48402cb3-9274-4da8-9b50-13e0a8b4b77e]\n18:29:26.109 [info] Complete prototype task [prototype task 2]\n18:29:26.109 [info] Complete prototype task [prototype task 4]\n18:29:26.109 [info] Process complete [two parallel routes process][08dba7e3-820e-4ea4-985e-5184b8041c80]\n```\n\nExamime the log messages. You should be able to see that two execution paths were proceeding in parallel.","ref":"task_types.html#parallel-and-prototype-task-example"},{"type":"extras","title":"Repeat Task Example - Mozart Task Types","doc":"The **Repeat Task** provides the ability to repeat a set of tasks as long a specified condition holds true.\n\nCopy the following new code into the MyBpmApplication module:\n\n```elixir\ndefmodule MyBpmApplication do\n  @moduledoc false\n  use Mozart.BpmProcess","ref":"task_types.html#repeat-task-example"},{"type":"extras","title":"Previous content here - Mozart Task Types","doc":"def continue(data) do\n    data.continue\n  end\n\n  defprocess \"repeat task process\" do\n    repeat_task \"repeat task\", &ME.continue/1 do\n      prototype_task(\"prototype task 1\")\n      prototype_task(\"prototype task 2\")\n      user_task(\"user task\", group: \"admin\")\n    end\n    prototype_task(\"last prototype task\")\n  end\n\nend\n\n```\n\nOpen an iex session and paste in the following:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nMyBpmApplication.load()\n{:ok, ppid, uid, business_key} = PE.start_process(\"repeat task process\", %{continue: true})\nPE.execute(ppid)\n\n```\n\nand you should see:\n\n```elixir\niex [09:10 :: 4] > alias Mozart.ProcessEngine, as: PE\nMozart.ProcessEngine\niex [09:10 :: 5] > alias Mozart.ProcessService, as: PS\nMozart.ProcessService\niex [09:10 :: 6] > MyBpmApplication.load()\n{:ok,\n [\"add x and y process\", \"one user task process\", \"two service tasks\",\n  \"subprocess task process\", \"two case process\",\n  \"receive barrower income process\", \"send barrower income process\",\n  \"two parallel routes process\", \"repeat task process\"]}\niex [09:10 :: 7] > {:ok, ppid, uid, business_key} = PE.start_process(\"repeat task process\", %{continue: true})\n\n09:11:31.919 [info] Start process instance [repeat task process][e3963cb8-410d-4039-ad81-eb6730808783]\n{:ok, #PID<0.300.0>, \"e3963cb8-410d-4039-ad81-eb6730808783\",\n \"ba5bded6-fbd9-4690-9643-453ab31457fa\"}\niex [09:10 :: 8] > PE.execute(ppid)\n:ok\n09:11:31.923 [info] New repeat task instance [repeat task][b600cf3d-8f84-4d16-8378-96f0bb152440]\n09:11:31.923 [info] New prototype task instance [prototype task 1][c6f6cfca-7097-4931-bea4-61404160e5d1]\n09:11:31.923 [info] Complete prototype task [prototype task 1]\n09:11:31.923 [info] New prototype task instance [prototype task 2][4c526ace-eb2b-4d7d-b918-83f1deda831f]\n09:11:31.923 [info] Complete prototype task [prototype task 2]\n09:11:31.924 [info] New user task instance [user task][2557b2e6-d6fa-4172-b25b-a809633e5217]\n```\n\nNow, let's complete the user task specifying **continue to be true***, i.e. %{continue: true}. This should cause the repeat tasks to be executed one more time:\n\n```elixir\nPS.complete_user_task(\"2557b2e6-d6fa-4172-b25b-a809633e5217\", %{continue: true})\n\n```\n\nThe output verifies this is the case:\n\n```elixir\niex [09:10 :: 9] > PS.complete_user_task(\"2557b2e6-d6fa-4172-b25b-a809633e5217\", %{continue: true})\n:ok\n09:17:20.481 [info] New prototype task instance [prototype task 1][4af087bb-865b-41b5-8d6b-55e6c9511d2b]\n09:17:20.481 [info] Complete user task [user task][2557b2e6-d6fa-4172-b25b-a809633e5217]\n09:17:20.481 [info] Complete prototype task [prototype task 1]\n09:17:20.481 [info] New prototype task instance [prototype task 2][965fa775-6280-43d0-ab50-8a966ed69dac]\n09:17:20.481 [info] Complete prototype task [prototype task 2]\n09:17:20.481 [info] New user task instance [user task][2e855696-e11d-42e7-89a9-330c2a87b11f]\n```\n\nNow, let's complete the new user task with **%{continue: false}**. This should cause the repeat to complete, the final **prototype_task** to complete and then the process to complete:\n\n```elixir\nPS.complete_user_task(\"2e855696-e11d-42e7-89a9-330c2a87b11f\", %{continue: false})\n\n```\n\nAnd we see the following, which verifies the expected behavior:\n\n```elixir\niex [09:10 :: 10] > PS.complete_user_task(\"2e855696-e11d-42e7-89a9-330c2a87b11f\", %{continue: false})\n:ok\n09:23:09.731 [info] Complete user task [user task][2e855696-e11d-42e7-89a9-330c2a87b11f]\n09:23:09.732 [info] Complete repeat task [repeat task]\n09:23:09.732 [info] New prototype task instance [last prototype task][5b6c63db-46bb-459b-a79f-d0c53a1662ca]\n09:23:09.732 [info] Complete prototype task [last prototype task]\n09:23:09.732 [info] Process complete [repeat task process][e3963cb8-410d-4039-ad81-eb6730808783]\n```","ref":"task_types.html#previous-content-here"},{"type":"extras","title":"Rule Task Example - Mozart Task Types","doc":"A **Rule Task** completes by evaluating a set of rules in a *rule table* and returning a derived value.\n\nCopy the following new code into the MyBpmApplication module:\n\n```elixir\ndefmodule MyBpmApplication do\n  @moduledoc false\n  use Mozart.BpmProcess","ref":"task_types.html#rule-task-example"},{"type":"extras","title":"Previous content here - Mozart Task Types","doc":"rule_table = \"\"\"\n  F     income      || status\n  1     > 50000     || approved\n  2     <= 49999    || declined\n  \"\"\"\n\n  defprocess \"single rule task process\" do\n    rule_task(\"loan decision\", inputs: \"income\", rule_table: rule_table)\n  end\n\n```\n\nOur table assumes there is a data property named **income**. If the value of *income* is greater than *50000*, the rule will return a property named **status** with a value of **approved**. If income is less than 50000, the value of *status* will be **declined**.\n\n[**Note**: Mozart uses the Tablex library for rule tasks. Tablex is very powerful. [Complete documentation is available in hexdocs](https://hexdocs.pm/tablex/readme.html).]\n\nOpen an iex session on your project and paste in:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nMyBpmApplication.load()\n{:ok, ppid, uid, business_key} = PE.start_process(\"single rule task process\", %{income: 60_000})\nPE.execute(ppid)\n\n```\n\nand you should see:\n\n```elixir\niex [16:31 :: 3] > MyBpmApplication.load()\n{:ok, .... deleted list of process names}\niex [16:31 :: 4] > {:ok, ppid, uid, business_key} = PE.start_process(\"single rule task process\", %{income: 60_000})\n16:31:29.857 [info] Start process instance [single rule task process][371b788e-33e6-4d55-9e0c-9e238f552b84]\n{:ok, #PID<0.299.0>, \"371b788e-33e6-4d55-9e0c-9e238f552b84\",\n \"83e7bb9a-a732-44b2-bdc3-ae4bbf6c1790\"}\niex [16:31 :: 5] > PE.execute(ppid)\n:ok\n16:31:29.860 [info] New rule task instance [loan decision][ab9ef596-0be3-44de-a66a-6fb41eaf0ab6]\n16:31:29.860 [info] Complete run task [loan decision[ab9ef596-0be3-44de-a66a-6fb41eaf0ab6]\n16:31:29.863 [info] Process complete [single rule task process][371b788e-33e6-4d55-9e0c-9e238f552b84]\n```\n\nNow let's look at the completed process data to see the property returned from the evaluation of the table:\n\n```elixir\niex [16:31 :: 6] > PS.get_completed_process_data(uid)\n%{status: \"approved\", income: 60000}\n```\n\nThe value of **status** is **\"approved\"**, as expected.","ref":"task_types.html#previous-content-here"},{"type":"extras","title":"Mozart Event Types","doc":"# Mozart Event Types\n\nCurrently, there is just one event type has been implemented (see below). This document will be updated as addtional event types are implemented.","ref":"event_types.html"},{"type":"extras","title":"Task Exit Event - Mozart Event Types","doc":"A **Task Exit Event** allows a process execution to exit an open task in response to an external event (a Phoenix.PubSub event). When the task exits in this way, process execution will follow an alternate execution path as specified the the *task exit event*.\n\nThe *task exit event* is implemented by the **defevent/3** DSL function which takes the following arguments:\n\n  * the name of the event\n  * **process**: the name of the process that the event will act upon\n  * **exit_task**: the name of the task to be exited\n  * **selector**: a function that matches on the target event\n  * **do**: one or more tasks to be executed when the target task is exited.\n  \n  Here is an example:\n\n  ```elixir\n  defevent \"exit loan decision 1\",\n    process: \"exit a user task 1\",\n    exit_task: \"user task 1\",\n    selector: &BpmAppWithEvent.event_selector/1 do\n      prototype_task(\"event 1 prototype task 1\")\n      prototype_task(\"event 1 prototype task 2\")\n  end\n  ```","ref":"event_types.html#task-exit-event"},{"type":"extras","title":"Task Exit Event Example - Mozart Event Types","doc":"In your MyBpmApplication module, enter the following code:\n\n```elixir\ndefmodule MyBpmApplication do\n  use Mozart.BpmProcess","ref":"event_types.html#task-exit-event-example"},{"type":"extras","title":"Previous Content Here - Mozart Event Types","doc":"","ref":"event_types.html#previous-content-here"},{"type":"extras","title":"Task Exit Event Example - Mozart Event Types","doc":"def exit_subprocess_task_event_selector(message) do\n    case message do\n      :exit_subprocess_task -> true\n      _ -> nil\n    end\n  end\n\n  defprocess \"exit a subprocess task\" do\n    subprocess_task(\"subprocess task\", process: \"subprocess process\")\n  end\n\n  defprocess \"subprocess process\" do\n    user_task(\"user task\", group: \"admin\")\n  end\n\n  defevent \"exit subprocess task\",\n    process: \"exit a subprocess task\",\n    exit_task: \"subprocess task\",\n    selector: &ME.exit_subprocess_task_event_selector/1 do\n      prototype_task(\"prototype task 1\")\n      prototype_task(\"prototype task 2\")\n  end\n\nend\n\n```\n\nNow start an iex session on your project and paste in the following:\n\n```elixir\nalias Mozart.ProcessEngine, as: PE\nalias Mozart.ProcessService, as: PS\nalias Phoenix.PubSub\nMyBpmApplication.load()\n{:ok, ppid, uid, business_key} = PE.start_process(\"exit a subprocess task\", %{})\nPE.execute(ppid)\n\n```\n\nAnd you should see the following:\n\n```elixir\niex [10:21 :: 1] > alias Mozart.ProcessEngine, as: PE\nMozart.ProcessEngine\niex [10:21 :: 2] > alias Mozart.ProcessService, as: PS\nMozart.ProcessService\niex [10:21 :: 3] > MyBpmApplication.load()\n{:ok,  }\niex [10:21 :: 4] > {:ok, ppid, uid, business_key} = PE.start_process(\"exit a subprocess task\", %{})\n10:21:26.446 [info] Start process instance [exit a subprocess task][847016f9-4818-4a30-9f65-3c9a5dcc1db5]\n{:ok, #PID<0.296.0>, \"847016f9-4818-4a30-9f65-3c9a5dcc1db5\",\n \"b085931c-89e5-48ca-ae81-e62ea486aef6\"}\niex [10:21 :: 5] > PE.execute(ppid)\n:ok\n10:21:26.450 [info] New subprocess task instance [subprocess task][01dd052c-6cf2-4b95-b2b8-69418f3aa332]\n10:21:26.450 [info] Start process instance [subprocess process][df6eca80-8c08-48ff-86cc-d825e8d7375f]\n10:21:26.450 [info] New user task instance [user task][827cb198-b40b-4e78-8c80-39a5cbe56a6e]\n```\n\nFrom the logs, we see that the subprocess started and a user task was opened.\n\nNow we will send an event that will cause the subprocess to exit. Additionally, the top level subprocess task will be completed and the tasks on the event will be executed:\n\n```elixir\nPubSub.broadcast(:pubsub, \"pe_topic\", {:event, :exit_subprocess_task})\n    \n```\n\nYou should see:\n\n```elixir\niex [10:21 :: 7] > PubSub.broadcast(:pubsub, \"pe_topic\", {:event, :exit_subprocess_task})\n:ok\n10:28:18.310 [info] New prototype task instance [prototype task 1][985d41a3-3e5c-469a-847b-7295a6d405c0]\n10:28:18.310 [info] Complete prototype task [prototype task 1]\n10:28:18.310 [info] New prototype task instance [prototype task 2][a6394485-3a7f-464f-9a47-3463b6bf87be]\n10:28:18.310 [info] Complete prototype task [prototype task 2]\n10:28:18.310 [info] Process complete [exit a subprocess task][847016f9-4818-4a30-9f65-3c9a5dcc1db5]\n```\n\nWhich indicates the expected behavior.","ref":"event_types.html#task-exit-event-example"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,51,46,48]}}